function [ PNarray ] = LFSR(Length, type)

% generating polynomial table: 'LFSR Reference -- M-Sequence, Linear ... '
% gold code pairs: 'preferred pairs m-sequences generation for gold codes'

if strcmp(type, 'bernoulli')
    PNarray=randsrc(1,Length);
elseif strcmp(type, 'legendre')
    if isprime(Length)~=1 || mod(Length, 4)~=3
        error('inappropriate length for legendre')
    end
    indexvector=mod(((1:(Length-1)/2).^2),Length);
    PNarray=zeros(1,Length);
    PNarray(indexvector+1)=1;
    PNarray=PNarray*2-1;
elseif strcmp(type, 'chu')
    R=25;
    while gcd(R,Length)~=1
        if R>=Length
            R=2;
        else
            R=R+1;
        end
    end
    PNarray=lteZadoffChuSeq(R,Length).';
else
    n=log2(Length+1);
    while 1
        start_state=(randsrc(1,n)+1)/2;
        if sum(start_state)~=0
            break
        end
    end
    state=start_state;
    PNarray=zeros(1, Length);
    index=0;
    switch n
        case 9
            GeneratingIndex=[9 5];
        case 8
            GeneratingIndex=[4 5 6 8]; %n=8
        case 7
             GeneratingIndex=[6 7]; %n=7
        case 6
            GeneratingIndex=[5 6];  %n=6
    end
    while index~=Length
        newcomp=mod(sum(state(GeneratingIndex)),2);
        state=[newcomp state(1:n-1)];
        
        index=index+1;
        PNarray(index)=newcomp;
    end
    
    if strcmp(type, 'complex')
        while 1
            imagSeq=(LFSR(Length, 'maximum')+1)/2;
            if sum(imagSeq==PNarray)~=Length
                break;
            end
        end
        PNarray=PNarray+sqrt(-1)*imagSeq;
    
    elseif strcmp(type, 'kasami')       %check this code is valid
        a=PNarray;
        q=2^(n/2)+1;
        Decind=mod(q:q:Length*q,Length);
        Decind(Decind==0)=Length;
        b=a(Decind);
        PNarray=mod(a+b,2);
    
    elseif strcmp(type, 'gold')
        k=2; % gcd(n,k)=2 when mod(n,4)=2
        q=2^k+1;
        Decind=mod(q:q:q*Length,Length);
        Decind(Decind==0)=Length;
        PNarray2=PNarray(Decind);
        PNarray=mod(PNarray+PNarray2, 2);
    elseif strcmp(type,'ternary')
        a=PNarray;a=-a;a=(a+1)/2;
        inb=mod(15*[0:Length-1],Length)+1;
        b=a(inb);
        Ac=zeros(Length,Length);
        for i=1:Length
            Ac(i,:)=circshift(a,[0 i-1]);
        end
        c=Ac*b.';
        v=unique(c);
        PNarray=(c-v(2))/(v(2)-v(1));
    elseif strcmp(type, 'maximum')
    else
        error('invalid sequence type');
    end
    PNarray=PNarray*2-1;
    if strcmp(type, 'complex')
        PNarray=PNarray-sqrt(-1);
    end
end
end
